#<profile4xi profile = "10" />
# -------------------------------------------------------------------------------------------------
#<program4xi program = "recv_handle4xi.py">
#<abstract4xi abstract = "providing function recv_handle4xi"/>
#<refer4xi reference = "reduction.py"/>
#<state4xi status = "ACK4tagging; ACK4moduletesting; NACK4systemtesting; [Rel 7.21]"/>
# -------------------------------------------------------------------------------------------------

# -------------------------------------------------------------------------------------------------
# Public API
# -------------------------------------------------------------------------------------------------
#<interface4xi interface = " 
__all__ = [ 'recv_handle4xi' ] 
#"
#/>

# -------------------------------------------------------------------------------------------------
# Standard Imports                          
# -------------------------------------------------------------------------------------------------
#<standardize4xi standard = "
try:                 # system ones
    import os
    import sys
    import signal
except:
    pass
try:                 # threading ones       
    import threading                
    import _thread   
except:
    pass
try:
    import _winapi
except:
    pass

                
try:                 # most-common ones
    import traceback         
except:
    pass
try:                 # rare ones
    import socket
except:
    pass 
 
#"
#/>

# -------------------------------------------------------------------------------------------------
# Multiprocessing Imports                          
# -------------------------------------------------------------------------------------------------
#<process4xi process = "
try:                             # Binary ones
    import _multiprocessing
except:
    pass
try:                             # Python ones
    from multiprocessing.reduction import recv_handle
except:
    print("recv_handle4xi.py: *** failed in from multiprocessing.reduction import recv_handle ***")
#"
#/>

# -------------------------------------------------------------------------------------------------
# DragonXi imports                              
# -------------------------------------------------------------------------------------------------
#<dragonize4xi dragonization = "
from constant_system4xi import get_windows_system4xi
from print4xi import print4xi
from time4xi import time4xi
#"
#/>
 
if sys.platform == get_windows_system4xi():  #dragonized
    
    # -------------------------------------------------------------------------------------------------
    #<function4xi function = "recv_handle4xi" usage = "
    #   rebuild_handle4xi.py:
    #       new_handle2xi = recv_handle4xi(conn2xi)
    #   reduction.py
    #       called in rebuild_handle as follows:
    #           new_handle = recv_handle(conn)
    #"> 
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
    # -------------------------------------------------------------------------------------------------
    def recv_handle4xi(conn):
        '''
        receives based on connection handle parameter
        '''
                
        # -----------------------------------------------------------------------------------------    
        # (Step 1/2)
        # ----------------------------------------------------------------------------------------- 
        class2print = "recv_handle4xi"
        method2print = "recv_handle4xi"
       
        message2print = "{(Step 1/2a) started" + "; recv-handle4xi-started: " + str(recv_handle4xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
            
        message2print = "{(Step 1/2b) passed" + "; conn-passed-to-recv-handle4xi: " + str(conn) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
                      
               
        # -----------------------------------------------------------------------------------------    
        # (Step 2/2)
        # ----------------------------------------------------------------------------------------- 
        message2print = "{(Step 2/2a) returning" + "; conn-recv-function-returning-by-recv-handle4xi: " + str(conn.recv) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)  
                
        return conn.recv().detach()                    
            
else:
    # -----------------------------------------------------------------------------------------
    # OSx receiving                              
    # -----------------------------------------------------------------------------------------
    # On MacOSX acknowledge receipt of fds -- see Issue14669
    ACKNOWLEDGE = sys.platform == 'darwin'
    def recv_handle4xi(conn):
        # -----------------------------------------------------------------------------------------    
        # (Step 1/2)
        # ----------------------------------------------------------------------------------------- 
        class2print = "recv_handle4xi"
        method2print = "recv_handle4xi"
       
        message2print = "{(Step 1/2a) started" + "; recv-handle4xi-started: " + str(recv_handle4xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
            
        message2print = "{(Step 1/2b) passed" + "; conn-passed-to-recv-handle4xi: " + str(conn) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
               
        # -----------------------------------------------------------------------------------------    
        # (Step 2/2)
        # -----------------------------------------------------------------------------------------             
        size2xi = struct.calcsize("@i")
        message2print = "{(Step 2/2a) set" + "; size2xi-set-by-recv-handle4xi to: " + str(size2xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 
        
        with socket.fromfd(conn.fileno(), socket.AF_UNIX, socket.SOCK_STREAM) as s:
            message2print = "{(Step 2/2b) using" + "; s-using-by-recv-handle4xi: " + str(s)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # 
            
            msg, ancdata, flags, addr = s.recvmsg(1, socket.CMSG_LEN(size2xi))
            
            message2print = "{(Step 2/2c) got" + "; msg-got-by-recv-handle4xi: " + str(msg)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # 
            
            message2print = "{(Step 2/2d) got" + "; ancdata-got-by-recv-handle4xi: " + str(ancdata)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # 
            
            message2print = "{(Step 2/2e) got" + "; flags-got-by-recv-handle4xi: " + str(flags)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # 
            
            message2print = "{(Step 2/2f) got" + "; addr-got-by-recv-handle4xi: " + str(addr)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # 
            
            try:
                if ACKNOWLEDGE:
                    conn.send_bytes(b'ACK')
                cmsg_level, cmsg_type, cmsg_data = ancdata[0]
                if (cmsg_level == socket.SOL_SOCKET and
                    cmsg_type == socket.SCM_RIGHTS):
                    return struct.unpack("@i", cmsg_data[:size2xi])[0]
            except (ValueError, IndexError, struct.error):
                pass
            raise RuntimeError('recv_handle4xi.py Invalid data received')
        
        
        
#</function4xi>
#</program4xi>

# -------------------------------------------------------------------------------------------------
# Recorded History                             
# ------------------------------------------------------------------------------------------------- 
#<record4xi record = "
#   test_none4xi.txt
#   sense-test-none4xi.xml
#   reason-test-none4xi.xml
#   ace-test-none4xi.xml
#"
#>

# -------------------------------------------------------------------------------------------------
# Legal Notice                             
# ------------------------------------------------------------------------------------------------- 
#<protect4xi protection = "copyright"
#   owner = "Small & Smart"
#   rights = "All rights reserved"
#   state = "California"
#   country = "United States of America"
#   references = "Legal notice at www.dragonxi.com; www.yourdragonxi.com; www.yourdragonxi.org; www.dragon2xi.com/; www.dragon4xi.com/; www.dragon8xi.com/ " 
#   codes = "Codes established in California and applicable to corporations in California"
#   compliance = "The California Law"
#   court = "Supreme Court of California, 350 McAllister Street, San Francisco, CA 94102-4783"
#/>
 

#<end4xi end = "
# -------------------------------------------------------------------------------------------------
# end of recv_handle4xi.py                             
# ------------------------------------------------------------------------------------------------- 
#"
#/>
